package com.hfkj.openapi.v1.model.request;

import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.DecimalMax;
import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.NotBlank;

/**
 * 请求分账
 * @className: RequestSeparateModel
 * @author: HuRui
 * @date: 2023/5/26
 **/
@Data
public class RequestSeparateCancelModel {

    /**
     * 商户号
     */
    @NotBlank(message = "商户号必填项")
    private String merchantNo;

    /**
     * 原分账指令流水号
     */
    @NotBlank(message = "原分账指令流水号必填项")
    @Length(min = 6, max = 50, message = "原分账指令流水号限制6至50位字符串长度")
    private String originOutSeparateNo;

    /**
     * 商户分账指令流水号
     */
    @NotBlank(message = "商户分账指令流水号必填项")
    @Length(min = 6, max = 50, message = "商户分账指令流水号限制6至50位字符串长度")
    private String outSeparateNo;

    /**
     * 撤销金额
     */
    @NotBlank(message = "撤销金额必填项")
    @DecimalMin(value = "0.01", message = "撤销金额最小0.01")
    @DecimalMax(value = "9999999", message = "撤销金额最大9999999")
    private String totalAmount;

    /**
     * 签名参数
     */
    @NotBlank(message = "签名必填项")
    private String sign;

}
